@@ -97,9 +97,22 @@ SIGNATURE_TREE_OA_SA_SV_TYPES_1 = SIGNATURE_TREE_OA_SA_SV.types[1]
 SIGNATURE_TREE_A_OA_SA_SV = get_signature_tree("a{oa{sa{sv}}}")
 SIGNATURE_TREE_A_OA_SA_SV_TYPES_0 = SIGNATURE_TREE_A_OA_SA_SV.types[0]
 
+TOKEN_B_AS_INT = ord("b")
+TOKEN_U_AS_INT = ord("u")
+TOKEN_Y_AS_INT = ord("y")
+TOKEN_A_AS_INT = ord("a")
 TOKEN_O_AS_INT = ord("o")
 TOKEN_S_AS_INT = ord("s")
 TOKEN_G_AS_INT = ord("g")
+TOKEN_N_AS_INT = ord("n")
+TOKEN_X_AS_INT = ord("x")
+TOKEN_T_AS_INT = ord("t")
+TOKEN_D_AS_INT = ord("d")
+TOKEN_Q_AS_INT = ord("q")
+TOKEN_V_AS_INT = ord("v")
+TOKEN_LEFT_CURLY_AS_INT = ord("{")
+TOKEN_LEFT_PAREN_AS_INT = ord("(")
+
 
 ARRAY = array.array
 SOL_SOCKET = socket.SOL_SOCKET
@@ -108,18 +121,19 @@ SCM_RIGHTS = socket.SCM_RIGHTS
 EAGAIN = errno.EAGAIN
 EWOULDBLOCK = errno.EWOULDBLOCK
 
-
-HEADER_MESSAGE_ARG_NAME = {
-    1: "path",
-    2: "interface",
-    3: "member",
-    4: "error_name",
-    5: "reply_serial",
-    6: "destination",
-    7: "sender",
-    8: "signature",
-    9: "unix_fds",
-}
+HEADER_IDX_TO_ARG_NAME = [
+    "",
+    "path",
+    "interface",
+    "member",
+    "error_name",
+    "reply_serial",
+    "destination",
+    "sender",
+    "signature",
+    "unix_fds",
+]
+HEADER_UNIX_FDS_IDX = HEADER_IDX_TO_ARG_NAME.index("unix_fds")
 
 _SignatureType = SignatureType
 _int = int
@@ -258,23 +272,17 @@ class Unmarshaller:
             self._buf = bytearray()
         else:
             del self._buf[:to_clear]
-        self._message = None
-        self._pos = 0
-        self._body_len = 0
-        self._serial = 0
-        self._header_len = 0
-        self._message_type = 0
-        self._flag = 0
-        self._msg_len = 0
-        self._is_native = 0
-        self._read_complete = False
+        self._msg_len = 0  # used to check if we have ready the header
+        self._read_complete = False  # used to check if we have ready the message
         # No need to reset the unpack functions, they are set in _read_header
         # every time a new message is processed.
 
     @property
     def message(self) -> Optional[Message]:
         """Return the message that has been unmarshalled."""
-        return self._message
+        if self._read_complete:
+            return self._message
+        return None
 
     def _has_another_message_in_buffer(self) -> bool:
         """Check if there is another message in the buffer."""
@@ -305,7 +313,7 @@ class Unmarshaller:
                 self._unix_fds.extend(
                     ARRAY("i", data[: len(data) - (len(data) % MAX_UNIX_FDS_SIZE)])
                 )
-        if msg == b"":
+        if not msg:
             raise EOFError()
         self._buf += msg
         if len(self._buf) < pos:
@@ -327,7 +335,7 @@ class Unmarshaller:
                 if errno == EAGAIN or errno == EWOULDBLOCK:
                     raise MARSHALL_STREAM_END_ERROR
                 raise
-            if data == b"":
+            if not data:
                 raise EOFError()
             self._buf += data
             if len(self._buf) >= pos:
@@ -338,7 +346,7 @@ class Unmarshaller:
         data = self._stream_reader(missing_bytes)  # type: ignore[misc]
         if data is None:
             raise MARSHALL_STREAM_END_ERROR
-        if data == b"":
+        if not data:
             raise EOFError()
         self._buf += data
         if len(self._buf) < pos:
@@ -437,44 +445,47 @@ class Unmarshaller:
 
     def _read_variant(self) -> Variant:
         signature = self._read_signature()
+        token_as_int = ord(signature[0])
         # verify in Variant is only useful on construction not unmarshalling
-        if signature == "n":
-            return Variant(SIGNATURE_TREE_N, self._read_int16_unpack(), False)
-        elif signature == "ay":
-            return Variant(
-                SIGNATURE_TREE_AY, self.read_array(SIGNATURE_TREE_AY_TYPES_0), False
-            )
-        elif signature == "a{qv}":
-            return Variant(
-                SIGNATURE_TREE_A_QV,
-                self.read_array(SIGNATURE_TREE_A_QV_TYPES_0),
-                False,
-            )
-        elif signature == "s":
-            return Variant(SIGNATURE_TREE_S, self._read_string_unpack(), False)
-        elif signature == "b":
-            return Variant(SIGNATURE_TREE_B, self._read_boolean(), False)
-        elif signature == "o":
-            return Variant(SIGNATURE_TREE_O, self._read_string_unpack(), False)
-        elif signature == "as":
-            return Variant(
-                SIGNATURE_TREE_AS, self.read_array(SIGNATURE_TREE_AS_TYPES_0), False
-            )
-        elif signature == "a{sv}":
-            return Variant(
-                SIGNATURE_TREE_A_SV,
-                self.read_array(SIGNATURE_TREE_A_SV_TYPES_0),
-                False,
-            )
-        elif signature == "ao":
-            return Variant(
-                SIGNATURE_TREE_AO, self.read_array(SIGNATURE_TREE_AO_TYPES_0), False
-            )
-        elif signature == "u":
-            return Variant(SIGNATURE_TREE_U, self._read_uint32_unpack(), False)
-        elif signature == "y":
-            self._pos += 1
-            return Variant(SIGNATURE_TREE_Y, self._buf[self._pos - 1], False)
+        if len(signature) == 1:
+            if token_as_int == TOKEN_N_AS_INT:
+                return Variant(SIGNATURE_TREE_N, self._read_int16_unpack(), False)
+            if token_as_int == TOKEN_S_AS_INT:
+                return Variant(SIGNATURE_TREE_S, self._read_string_unpack(), False)
+            if token_as_int == TOKEN_B_AS_INT:
+                return Variant(SIGNATURE_TREE_B, self._read_boolean(), False)
+            if token_as_int == TOKEN_O_AS_INT:
+                return Variant(SIGNATURE_TREE_O, self._read_string_unpack(), False)
+            if token_as_int == TOKEN_U_AS_INT:
+                return Variant(SIGNATURE_TREE_U, self._read_uint32_unpack(), False)
+            if token_as_int == TOKEN_Y_AS_INT:
+                self._pos += 1
+                return Variant(SIGNATURE_TREE_Y, self._buf[self._pos - 1], False)
+        elif token_as_int == TOKEN_A_AS_INT:
+            if signature == "ay":
+                return Variant(
+                    SIGNATURE_TREE_AY, self.read_array(SIGNATURE_TREE_AY_TYPES_0), False
+                )
+            if signature == "a{qv}":
+                return Variant(
+                    SIGNATURE_TREE_A_QV,
+                    self.read_array(SIGNATURE_TREE_A_QV_TYPES_0),
+                    False,
+                )
+            if signature == "as":
+                return Variant(
+                    SIGNATURE_TREE_AS, self.read_array(SIGNATURE_TREE_AS_TYPES_0), False
+                )
+            if signature == "a{sv}":
+                return Variant(
+                    SIGNATURE_TREE_A_SV,
+                    self.read_array(SIGNATURE_TREE_A_SV_TYPES_0),
+                    False,
+                )
+            if signature == "ao":
+                return Variant(
+                    SIGNATURE_TREE_AO, self.read_array(SIGNATURE_TREE_AO_TYPES_0), False
+                )
         tree = get_signature_tree(signature)
         signature_type = tree.types[0]
         return Variant(
@@ -510,46 +521,61 @@ class Unmarshaller:
         else:
             array_length = self._uint32_unpack(self._buf, self._pos - UINT32_SIZE)[0]  # type: ignore[misc]
 
-        child_type = type_.children[0]
-        token = child_type.token
+        child_type: SignatureType = type_.children[0]
+        token_as_int = ord(child_type.token[0])
 
-        if token in "xtd{(":
+        if (
+            token_as_int == TOKEN_X_AS_INT
+            or token_as_int == TOKEN_T_AS_INT
+            or token_as_int == TOKEN_D_AS_INT
+            or token_as_int == TOKEN_LEFT_CURLY_AS_INT
+            or token_as_int == TOKEN_LEFT_PAREN_AS_INT
+        ):
             # the first alignment is not included in the array size
             self._pos += -self._pos & 7  # align 8
 
-        if token == "y":
+        if token_as_int == TOKEN_Y_AS_INT:
             self._pos += array_length
             return self._buf[self._pos - array_length : self._pos]
 
-        if token == "{":
+        if token_as_int == TOKEN_LEFT_CURLY_AS_INT:
             result_dict: Dict[Any, Any] = {}
             beginning_pos = self._pos
             children = child_type.children
             child_0 = children[0]
             child_1 = children[1]
-            child_0_token = child_0.token
-            child_1_token = child_1.token
+            child_0_token_as_int = ord(child_0.token[0])
+            child_1_token_as_int = ord(child_1.token[0])
             # Strings with variant values are the most common case
             # so we optimize for that by inlining the string reading
             # and the variant reading here
-            if child_0_token in "os" and child_1_token == "v":
+            if (
+                child_0_token_as_int == TOKEN_O_AS_INT
+                or child_0_token_as_int == TOKEN_S_AS_INT
+            ) and child_1_token_as_int == TOKEN_V_AS_INT:
                 while self._pos - beginning_pos < array_length:
                     self._pos += -self._pos & 7  # align 8
                     key: Union[str, int] = self._read_string_unpack()
                     result_dict[key] = self._read_variant()
-            elif child_0_token == "q" and child_1_token == "v":
+            elif (
+                child_0_token_as_int == TOKEN_Q_AS_INT
+                and child_1_token_as_int == TOKEN_V_AS_INT
+            ):
                 while self._pos - beginning_pos < array_length:
                     self._pos += -self._pos & 7  # align 8
                     key = self._read_uint16_unpack()
                     result_dict[key] = self._read_variant()
-            elif child_0_token in "os" and child_1_token == "a":
+            if (
+                child_0_token_as_int == TOKEN_O_AS_INT
+                or child_0_token_as_int == TOKEN_S_AS_INT
+            ) and child_1_token_as_int == TOKEN_A_AS_INT:
                 while self._pos - beginning_pos < array_length:
                     self._pos += -self._pos & 7  # align 8
                     key = self._read_string_unpack()
                     result_dict[key] = self.read_array(child_1)
             else:
-                reader_1 = self._readers[child_1_token]
-                reader_0 = self._readers[child_0_token]
+                reader_1 = self._readers[child_1.token]
+                reader_0 = self._readers[child_0.token]
                 while self._pos - beginning_pos < array_length:
                     self._pos += -self._pos & 7  # align 8
                     key = reader_0(self, child_0)
@@ -562,11 +588,11 @@ class Unmarshaller:
 
         result_list = []
         beginning_pos = self._pos
-        if token in "os":
+        if token_as_int == TOKEN_O_AS_INT or token_as_int == TOKEN_S_AS_INT:
             while self._pos - beginning_pos < array_length:
                 result_list.append(self._read_string_unpack())
             return result_list
-        reader = self._readers[token]
+        reader = self._readers[child_type.token]
         while self._pos - beginning_pos < array_length:
             result_list.append(reader(self, child_type))
         return result_list
@@ -587,9 +613,11 @@ class Unmarshaller:
             signature_len = buf[self._pos]  # byte
             o = self._pos + 1
             self._pos += signature_len + 2  # one for the byte, one for the '\0'
+            if field_0 == HEADER_UNIX_FDS_IDX:  # defined by self._unix_fds
+                continue
             token_as_int = buf[o]
             # Now that we have the token we can read the variant value
-            key = HEADER_MESSAGE_ARG_NAME[field_0]
+            key = HEADER_IDX_TO_ARG_NAME[field_0]
             # Strings and signatures are the most common types
             # so we inline them for performance
             if token_as_int == TOKEN_O_AS_INT or token_as_int == TOKEN_S_AS_INT:
@@ -625,26 +653,26 @@ class Unmarshaller:
         ):
             self._is_native = 1  # pragma: no cover
             self._body_len = _cast_uint32_native(  # type: ignore[name-defined] # pragma: no cover
-                self._buf, 4
+                buffer, 4
             )
             self._serial = _cast_uint32_native(  # type: ignore[name-defined] # pragma: no cover
-                self._buf, 8
+                buffer, 8
             )
             self._header_len = _cast_uint32_native(  # type: ignore[name-defined] # pragma: no cover
-                self._buf, 12
+                buffer, 12
             )
         elif endian == LITTLE_ENDIAN:
             (
                 self._body_len,
                 self._serial,
                 self._header_len,
-            ) = UNPACK_HEADER_LITTLE_ENDIAN(self._buf, 4)
+            ) = UNPACK_HEADER_LITTLE_ENDIAN(buffer, 4)
             self._uint32_unpack = UINT32_UNPACK_LITTLE_ENDIAN
             self._int16_unpack = INT16_UNPACK_LITTLE_ENDIAN
             self._uint16_unpack = UINT16_UNPACK_LITTLE_ENDIAN
         elif endian == BIG_ENDIAN:
             self._body_len, self._serial, self._header_len = UNPACK_HEADER_BIG_ENDIAN(
-                self._buf, 4
+                buffer, 4
             )
             self._uint32_unpack = UINT32_UNPACK_BIG_ENDIAN
             self._int16_unpack = INT16_UNPACK_BIG_ENDIAN
@@ -667,42 +695,47 @@ class Unmarshaller:
         self._pos = HEADER_ARRAY_OF_STRUCT_SIGNATURE_POSITION
         header_fields = self._header_fields(self._header_len)
         self._pos += -self._pos & 7  # align 8
-        header_fields.pop("unix_fds", None)  # defined by self._unix_fds
         signature = header_fields.pop("signature", "")
         if not self._body_len:
             tree = SIGNATURE_TREE_EMPTY
             body: List[Any] = []
-        elif signature == "s":
-            tree = SIGNATURE_TREE_S
-            body = [self._read_string_unpack()]
-        elif signature == "sa{sv}as":
-            tree = SIGNATURE_TREE_SA_SV_AS
-            body = [
-                self._read_string_unpack(),
-                self.read_array(SIGNATURE_TREE_SA_SV_AS_TYPES_1),
-                self.read_array(SIGNATURE_TREE_SA_SV_AS_TYPES_2),
-            ]
-        elif signature == "oa{sa{sv}}":
-            tree = SIGNATURE_TREE_OA_SA_SV
-            body = [
-                self._read_string_unpack(),
-                self.read_array(SIGNATURE_TREE_OA_SA_SV_TYPES_1),
-            ]
-        elif signature == "oas":
-            tree = SIGNATURE_TREE_OAS
-            body = [
-                self._read_string_unpack(),
-                self.read_array(SIGNATURE_TREE_OAS_TYPES_1),
-            ]
-        elif signature == "a{oa{sa{sv}}}":
-            tree = SIGNATURE_TREE_A_OA_SA_SV
-            body = [self.read_array(SIGNATURE_TREE_A_OA_SA_SV_TYPES_0)]
-        elif signature == "o":
-            tree = SIGNATURE_TREE_O
-            body = [self._read_string_unpack()]
         else:
-            tree = get_signature_tree(signature)
-            body = [self._readers[t.token](self, t) for t in tree.types]
+            token_as_int = ord(signature[0])
+            if len(signature) == 1:
+                if token_as_int == TOKEN_O_AS_INT:
+                    tree = SIGNATURE_TREE_O
+                    body = [self._read_string_unpack()]
+                elif token_as_int == TOKEN_S_AS_INT:
+                    tree = SIGNATURE_TREE_S
+                    body = [self._read_string_unpack()]
+                else:
+                    tree = get_signature_tree(signature)
+                    body = [self._readers[t.token](self, t) for t in tree.types]
+            elif token_as_int == TOKEN_S_AS_INT and signature == "sa{sv}as":
+                tree = SIGNATURE_TREE_SA_SV_AS
+                body = [
+                    self._read_string_unpack(),
+                    self.read_array(SIGNATURE_TREE_SA_SV_AS_TYPES_1),
+                    self.read_array(SIGNATURE_TREE_SA_SV_AS_TYPES_2),
+                ]
+            elif token_as_int == TOKEN_O_AS_INT and signature == "oa{sa{sv}}":
+                tree = SIGNATURE_TREE_OA_SA_SV
+                body = [
+                    self._read_string_unpack(),
+                    self.read_array(SIGNATURE_TREE_OA_SA_SV_TYPES_1),
+                ]
+            elif token_as_int == TOKEN_O_AS_INT and signature == "oas":
+                tree = SIGNATURE_TREE_OAS
+                body = [
+                    self._read_string_unpack(),
+                    self.read_array(SIGNATURE_TREE_OAS_TYPES_1),
+                ]
+            elif token_as_int == TOKEN_A_AS_INT and signature == "a{oa{sa{sv}}}":
+                tree = SIGNATURE_TREE_A_OA_SA_SV
+                body = [self.read_array(SIGNATURE_TREE_A_OA_SA_SV_TYPES_0)]
+            else:
+                tree = get_signature_tree(signature)
+                body = [self._readers[t.token](self, t) for t in tree.types]
 
         flags = MESSAGE_FLAG_MAP.get(self._flag)
         if flags is None:
