@@ -3,7 +3,7 @@ import logging
 import socket
 import traceback
 import xml.etree.ElementTree as ET
-from types import TracebackType
+from functools import partial
 from typing import Any, Callable, Dict, List, Optional, Type, Union
 
 from . import introspection as intr
@@ -21,6 +21,7 @@ from .constants import (
 from .errors import DBusError, InvalidAddressError
 from .message import Message
 from .proxy_object import BaseProxyObject
+from .send_reply import SendReply
 from .service import ServiceInterface, _Method
 from .signature import Variant
 from .validators import assert_bus_name_valid, assert_object_path_valid
@@ -28,6 +29,8 @@ from .validators import assert_bus_name_valid, assert_object_path_valid
 MESSAGE_TYPE_CALL = MessageType.METHOD_CALL
 MESSAGE_TYPE_SIGNAL = MessageType.SIGNAL
 NO_REPLY_EXPECTED_VALUE = MessageFlag.NO_REPLY_EXPECTED.value
+NO_REPLY_EXPECTED = MessageFlag.NO_REPLY_EXPECTED
+NONE = MessageFlag.NONE
 _LOGGER = logging.getLogger(__name__)
 
 
@@ -36,7 +39,13 @@ _Message = Message
 
 def _expects_reply(msg: _Message) -> bool:
     """Whether a message expects a reply."""
-    return not (msg.flags.value & NO_REPLY_EXPECTED_VALUE)
+    if msg.flags is NO_REPLY_EXPECTED:
+        return False
+    if msg.flags is NONE:
+        return True
+    # Slow check for NO_REPLY_EXPECTED
+    flag_value = msg.flags.value
+    return not (flag_value & NO_REPLY_EXPECTED_VALUE)
 
 
 def _block_unexpected_reply(reply: _Message) -> None:
@@ -57,55 +66,6 @@ def _block_unexpected_reply(reply: _Message) -> None:
 BLOCK_UNEXPECTED_REPLY = _block_unexpected_reply
 
 
-class SendReply:
-    """A context manager to send a reply to a message."""
-
-    __slots__ = ("_bus", "_msg")
-
-    def __init__(self, bus: "BaseMessageBus", msg: Message) -> None:
-        """Create a new reply context manager."""
-        self._bus = bus
-        self._msg = msg
-
-    def __enter__(self):
-        return self
-
-    def __call__(self, reply: Message) -> None:
-        self._bus.send(reply)
-
-    def _exit(
-        self,
-        exc_type: Optional[Type[Exception]],
-        exc_value: Optional[Exception],
-        tb: Optional[TracebackType],
-    ) -> bool:
-        if exc_value:
-            if isinstance(exc_value, DBusError):
-                self(exc_value._as_message(self._msg))
-            else:
-                self(
-                    Message.new_error(
-                        self._msg,
-                        ErrorType.SERVICE_ERROR,
-                        f"The service interface raised an error: {exc_value}.\n{traceback.format_tb(tb)}",
-                    )
-                )
-            return True
-
-        return False
-
-    def __exit__(
-        self,
-        exc_type: Optional[Type[Exception]],
-        exc_value: Optional[Exception],
-        tb: Optional[TracebackType],
-    ) -> bool:
-        return self._exit(exc_type, exc_value, tb)
-
-    def send_error(self, exc: Exception) -> None:
-        self._exit(exc.__class__, exc, exc.__traceback__)
-
-
 class BaseMessageBus:
     """An abstract class to manage a connection to a DBus message bus.
 
@@ -202,6 +162,9 @@ class BaseMessageBus:
 
         # machine id is lazy loaded
         self._machine_id: Optional[int] = None
+        self._sock: Optional[socket.socket] = None
+        self._fd: Optional[int] = None
+        self._stream: Optional[Any] = None
 
         self._setup_socket()
 
@@ -298,6 +261,7 @@ class BaseMessageBus:
         bus_name: str,
         path: str,
         callback: Callable[[Optional[intr.Node], Optional[Exception]], None],
+        check_callback_type: bool = True,
     ) -> None:
         """Get introspection data for the node at the given path from the given
         bus name.
@@ -317,7 +281,8 @@ class BaseMessageBus:
             - :class:`InvalidObjectPathError <dbus_fast.InvalidObjectPathError>` - If the given object path is not valid.
             - :class:`InvalidBusNameError <dbus_fast.InvalidBusNameError>` - If the given bus name is not valid.
         """
-        BaseMessageBus._check_callback_type(callback)
+        if check_callback_type:
+            BaseMessageBus._check_callback_type(callback)
 
         def reply_notify(reply: Optional[Message], err: Optional[Exception]) -> None:
             try:
@@ -416,6 +381,7 @@ class BaseMessageBus:
         callback: Optional[
             Callable[[Optional[RequestNameReply], Optional[Exception]], None]
         ] = None,
+        check_callback_type: bool = True,
     ) -> None:
         """Request that this message bus owns the given name.
 
@@ -432,7 +398,7 @@ class BaseMessageBus:
         """
         assert_bus_name_valid(name)
 
-        if callback is not None:
+        if callback is not None and check_callback_type:
             BaseMessageBus._check_callback_type(callback)
 
         if type(flags) is not NameFlag:
@@ -469,6 +435,7 @@ class BaseMessageBus:
         callback: Optional[
             Callable[[Optional[ReleaseNameReply], Optional[Exception]], None]
         ] = None,
+        check_callback_type: bool = True,
     ) -> None:
         """Request that this message bus release the given name.
 
@@ -484,7 +451,7 @@ class BaseMessageBus:
         """
         assert_bus_name_valid(name)
 
-        if callback is not None:
+        if callback is not None and check_callback_type:
             BaseMessageBus._check_callback_type(callback)
 
         message = Message(
@@ -636,7 +603,7 @@ class BaseMessageBus:
 
         self._method_return_handlers.clear()
 
-        for path in list(self._path_exports.keys()):
+        for path in list(self._path_exports):
             self.unexport(path)
 
         self._user_message_handlers.clear()
@@ -758,38 +725,39 @@ class BaseMessageBus:
         if err:
             raise err
 
+    def _reply_notify(
+        self,
+        msg: Message,
+        callback: Optional[Callable[[Optional[Message], Optional[Exception]], None]],
+        reply: Optional[Message],
+        err: Optional[Exception],
+    ) -> None:
+        """Callback on reply."""
+        if reply and msg.destination and reply.sender:
+            self._name_owners[msg.destination] = reply.sender
+        callback(reply, err)
+
     def _call(
         self,
         msg: Message,
         callback: Optional[Callable[[Optional[Message], Optional[Exception]], None]],
-        check_callback: bool = True,
     ) -> None:
-        if check_callback:
-            BaseMessageBus._check_callback_type(callback)
-
         if not msg.serial:
             msg.serial = self.next_serial()
 
-        no_reply_expected = not _expects_reply(msg)
+        reply_expected = _expects_reply(msg)
         # Make sure the return reply handler is installed
         # before sending the message to avoid a race condition
         # where the reply is lost in case the backend can
         # send it right away.
-        if not no_reply_expected:
-
-            def _reply_notify(
-                reply: Optional[Message], err: Optional[Exception]
-            ) -> None:
-                """Callback on reply."""
-                if reply and msg.destination and reply.sender:
-                    self._name_owners[msg.destination] = reply.sender
-                callback(reply, err)
-
-            self._method_return_handlers[msg.serial] = _reply_notify
+        if reply_expected:
+            self._method_return_handlers[msg.serial] = partial(
+                self._reply_notify, msg, callback
+            )
 
         self.send(msg)
 
-        if no_reply_expected:
+        if not reply_expected:
             callback(None, None)
 
     @staticmethod
@@ -827,6 +795,7 @@ class BaseMessageBus:
             )
 
     def _process_message(self, msg: _Message) -> None:
+        """Process a message received from the message bus."""
         handled = False
         for user_handler in self._user_message_handlers:
             try:
@@ -842,13 +811,9 @@ class BaseMessageBus:
                     handled = True
                     break
                 else:
-                    logging.error(
-                        f"A message handler raised an exception: {e}.\n{traceback.format_exc()}"
-                    )
+                    logging.exception("A message handler raised an exception: %s", e)
             except Exception as e:
-                logging.error(
-                    f"A message handler raised an exception: {e}.\n{traceback.format_exc()}"
-                )
+                logging.exception("A message handler raised an exception: %s", e)
                 if msg.message_type is MESSAGE_TYPE_CALL:
                     self.send(
                         Message.new_error(
@@ -877,7 +842,7 @@ class BaseMessageBus:
         if msg.message_type is MESSAGE_TYPE_CALL:
             if not handled:
                 handler = self._find_message_handler(msg)
-                if not _expects_reply(msg):
+                if _expects_reply(msg) is False:
                     if handler:
                         handler(msg, BLOCK_UNEXPECTED_REPLY)
                     else:
@@ -898,79 +863,77 @@ class BaseMessageBus:
                             Message.new_error(
                                 msg,
                                 ErrorType.UNKNOWN_METHOD,
-                                f'{msg.interface}.{msg.member} with signature "{msg.signature}" could not be found',
+                                f"{msg.interface}.{msg.member} with signature "
+                                f'"{msg.signature}" could not be found',
                             )
                         )
             return
 
         # An ERROR or a METHOD_RETURN
-        if msg.reply_serial in self._method_return_handlers:
+        return_handler = self._method_return_handlers.get(msg.reply_serial)
+        if return_handler is not None:
             if not handled:
-                return_handler = self._method_return_handlers[msg.reply_serial]
                 return_handler(msg, None)
             del self._method_return_handlers[msg.reply_serial]
 
+    def _callback_method_handler(
+        self,
+        interface: ServiceInterface,
+        method: _Method,
+        msg: Message,
+        send_reply: Callable[[Message], None],
+    ) -> None:
+        """This is the callback that will be called when a method call is."""
+        args = ServiceInterface._c_msg_body_to_args(msg) if msg.unix_fds else msg.body
+        result = method.fn(interface, *args)
+        if send_reply is BLOCK_UNEXPECTED_REPLY or _expects_reply(msg) is False:
+            return
+        body, fds = ServiceInterface._c_fn_result_to_body(
+            result,
+            signature_tree=method.out_signature_tree,
+            replace_fds=self._negotiate_unix_fd,
+        )
+        send_reply(
+            Message(
+                message_type=MessageType.METHOD_RETURN,
+                reply_serial=msg.serial,
+                destination=msg.sender,
+                signature=method.out_signature,
+                body=body,
+                unix_fds=fds,
+            )
+        )
+
     def _make_method_handler(
         self, interface: ServiceInterface, method: _Method
     ) -> Callable[[Message, Callable[[Message], None]], None]:
-        method_fn = method.fn
-        out_signature_tree = method.out_signature_tree
-        negotiate_unix_fd = self._negotiate_unix_fd
-        out_signature = method.out_signature
-        message_type_method_return = MessageType.METHOD_RETURN
-        msg_body_to_args = ServiceInterface._msg_body_to_args
-        fn_result_to_body = ServiceInterface._fn_result_to_body
-
-        def _callback_method_handler(
-            msg: Message, send_reply: Callable[[Message], None]
-        ) -> None:
-            """This is the callback that will be called when a method call is."""
-            args = msg_body_to_args(msg) if msg.unix_fds else msg.body
-            result = method_fn(interface, *args)
-            if send_reply is BLOCK_UNEXPECTED_REPLY or not _expects_reply(msg):
-                return
-            body, fds = fn_result_to_body(
-                result,
-                signature_tree=out_signature_tree,
-                replace_fds=negotiate_unix_fd,
-            )
-            send_reply(
-                Message(
-                    message_type=message_type_method_return,
-                    reply_serial=msg.serial,
-                    destination=msg.sender,
-                    signature=out_signature,
-                    body=body,
-                    unix_fds=fds,
-                )
-            )
-
-        return _callback_method_handler
+        return partial(self._callback_method_handler, interface, method)
 
     def _find_message_handler(
         self, msg: _Message
     ) -> Optional[Callable[[Message, Callable[[Message], None]], None]]:
-        if (
-            msg.interface == "org.freedesktop.DBus.Introspectable"
-            and msg.member == "Introspect"
-            and msg.signature == ""
-        ):
-            return self._default_introspect_handler
+        if "org.freedesktop.DBus." in msg.interface:
+            if (
+                msg.interface == "org.freedesktop.DBus.Introspectable"
+                and msg.member == "Introspect"
+                and msg.signature == ""
+            ):
+                return self._default_introspect_handler
 
-        if msg.interface == "org.freedesktop.DBus.Properties":
-            return self._default_properties_handler
+            if msg.interface == "org.freedesktop.DBus.Properties":
+                return self._default_properties_handler
 
-        if msg.interface == "org.freedesktop.DBus.Peer":
-            if msg.member == "Ping" and msg.signature == "":
-                return self._default_ping_handler
-            elif msg.member == "GetMachineId" and msg.signature == "":
-                return self._default_get_machine_id_handler
+            if msg.interface == "org.freedesktop.DBus.Peer":
+                if msg.member == "Ping" and msg.signature == "":
+                    return self._default_ping_handler
+                elif msg.member == "GetMachineId" and msg.signature == "":
+                    return self._default_get_machine_id_handler
 
-        if (
-            msg.interface == "org.freedesktop.DBus.ObjectManager"
-            and msg.member == "GetManagedObjects"
-        ):
-            return self._default_get_managed_objects_handler
+            if (
+                msg.interface == "org.freedesktop.DBus.ObjectManager"
+                and msg.member == "GetManagedObjects"
+            ):
+                return self._default_get_managed_objects_handler
 
         msg_path = msg.path
         if msg_path:
@@ -978,7 +941,7 @@ class BaseMessageBus:
             if not interfaces:
                 return None
             for interface in interfaces:
-                methods = ServiceInterface._get_methods(interface)
+                methods = ServiceInterface._c_get_methods(interface)
                 for method in methods:
                     if method.disabled:
                         continue
@@ -988,7 +951,7 @@ class BaseMessageBus:
                         and msg.member == method.name
                         and msg.signature == method.in_signature
                     ):
-                        return ServiceInterface._get_handler(interface, method, self)
+                        return ServiceInterface._c_get_handler(interface, method, self)
 
         return None
 
@@ -1033,7 +996,6 @@ class BaseMessageBus:
                 member="GetMachineId",
             ),
             reply_handler,
-            check_callback=False,
         )
 
     def _default_get_managed_objects_handler(
@@ -1245,7 +1207,7 @@ class BaseMessageBus:
                 logging.error(
                     f'add match request failed. match="{self._name_owner_match_rule}", {err}'
                 )
-            if msg.message_type == MessageType.ERROR:
+            elif msg.message_type == MessageType.ERROR:
                 logging.error(
                     f'add match request failed. match="{self._name_owner_match_rule}", {msg.body[0]}'
                 )
@@ -1260,7 +1222,6 @@ class BaseMessageBus:
                 body=[self._name_owner_match_rule],
             ),
             add_match_notify,
-            check_callback=False,
         )
 
     def _add_match_rule(self, match_rule):
@@ -1279,7 +1240,7 @@ class BaseMessageBus:
         def add_match_notify(msg: Message, err: Optional[Exception]) -> None:
             if err:
                 logging.error(f'add match request failed. match="{match_rule}", {err}')
-            if msg.message_type == MessageType.ERROR:
+            elif msg.message_type == MessageType.ERROR:
                 logging.error(
                     f'add match request failed. match="{match_rule}", {msg.body[0]}'
                 )
@@ -1294,7 +1255,6 @@ class BaseMessageBus:
                 body=[match_rule],
             ),
             add_match_notify,
-            check_callback=False,
         )
 
     def _remove_match_rule(self, match_rule):
@@ -1318,7 +1278,7 @@ class BaseMessageBus:
                 logging.error(
                     f'remove match request failed. match="{match_rule}", {err}'
                 )
-            if msg.message_type == MessageType.ERROR:
+            elif msg.message_type == MessageType.ERROR:
                 logging.error(
                     f'remove match request failed. match="{match_rule}", {msg.body[0]}'
                 )
@@ -1333,5 +1293,4 @@ class BaseMessageBus:
                 body=[match_rule],
             ),
             remove_match_notify,
-            check_callback=False,
         )
