@@ -3,10 +3,20 @@ import os
 
 import pytest
 
-from dbus_fast.auth import UID_NOT_SPECIFIED, AuthExternal
+from dbus_fast.auth import (
+    UID_NOT_SPECIFIED,
+    AuthAnnonymous,
+    AuthAnonymous,
+    AuthExternal,
+)
 from dbus_fast.errors import AuthError
 
 
+def test_annonymous_backcompat():
+    auth = AuthAnnonymous()
+    assert isinstance(auth, AuthAnonymous)
+
+
 def test_uid_is_set():
     auth = AuthExternal(uid=999)
     assert auth._authentication_start() == "AUTH EXTERNAL 393939"
